package waffle.com;

public class MathVectors {
    /**
     * Returns the sum of two vectors of the same dimension.
     * 
     * @param vector1
     * @param vector2
     * @return
     */
    public static double[] add(double[] vector1, double[] vector2) {
        // Returns an empty vector if we send incorrect input
        if (vector1.length != vector2.length) {
            return new double[0];
        }

        double[] result = new double[vector1.length];

        for (int i = 0; i < vector1.length; i++) {
            result[i] = vector1[i] + vector2[i];
        }

        return result;
    }

    /**
     * Returns the product of a vector and a scalar.
     * 
     * @param vector
     * @param scalar
     * @return
     */
    public static double[] mul(double[] vector, double scalar) {
        double[] result = new double[vector.length];

        for (int i = 0; i < vector.length; i++) {
            result[i] = vector[i] * scalar;
        }

        return result;
    }

    /**
     * Returns the difference of two vectors of the same dimension.
     * 
     * @param vector1
     * @param vector2
     * @return
     */
    public static double[] sub(double[] vector1, double[] vector2) {
        // Returns an empty vector if we send incorrect input
        if (vector1.length != vector2.length) {
            return new double[0];
        }

        // To substract, we add the inverse of the second matrix
        double[] result = add(vector1, mul(vector2, -1.0));

        return result;
    }

    /**
     * Returns the dimension of a given vector.
     * 
     * @param vector
     * @return
     */
    public static int len(double[] vector) {
        return vector.length;
    }

    public static double norm(double[] vector) {
        double result = 0.0;

        for (int i = 0; i < vector.length; i++) {
            result += vector[i] * vector[i];
        }

        return Math.sqrt(result);
    }
}
