package ch.hepia.exercice_3_1;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class Container {
    private int capacity;
    private int amount;

    public Container(int capacity) {
        this.capacity = capacity;
        this.amount = 0;
    }

    public static Container withCapacity(int capacity){
        return new Container(capacity);
    }

    public void fillToTheMax(){
        this.amount = this.capacity;
    }

    public void fillWith(int amount){
        if (amount < 0){
            throw new IllegalArgumentException("Amount to add cannot be negative");
        }

        this.amount = (this.amount + amount >= this.capacity) ? this.capacity : this.amount + amount;
    }

    public void remove(int amount){
        if (amount < 0){
            throw new IllegalArgumentException("Amount to substract cannot be negative");
        }

        this.amount = (this.amount - amount >= 0) ? this.amount - amount : 0;
    }

    /**
     * Fill a single container using the remaining amount
     * @param recipient
     */
    public void fillTo(Container recipient){
        // Fill the container to their max capacity if possible, else to the remaining amount
        int amountToTransfer = (this.quantity() >= recipient.remaining()) ? recipient.remaining() : this.quantity();
        recipient.fillWith(amountToTransfer);
        // Limit the remaining amount from going below zero while substracting
        this.remove(amountToTransfer);
    }

    /**
     * Fill one or more containers using the remaining amount
     * @param recipient [REQUIRED] the first container
     * @param recipients one or more containers to fill
     */
    public void fillTo(Container recipient, Container... recipients){
        List<Container> containers = new ArrayList<>();
        // Concatenate our first required parameter and the arbiritrary amount of parameters into a list
        containers.add(recipient);
        Arrays.stream(recipients).forEach(item -> containers.add(item));

        for (Container container : containers) {
            this.fillTo(container);
        }
    }

    public boolean isFull(){
        return (this.amount == this.capacity) ? true : false;
    }

    public int quantity(){
        return this.amount;
    }

    public void flush(){
        this.amount = 0;
    }

    public int remaining(){
        return this.capacity - this.amount;
    }
}
