package ch.hepia.accounts;

import java.util.Objects;

public class Account {

    private String owner;
    private int amount;
    private final int AMOUNT_LIMIT = -2000;


    public Account(String firstName, String lastName) {
        this.owner = firstName + " " + lastName;
    }

    public Account(String firstName, String lastName, int amount) {
        this(firstName, lastName);
        this.deposit(amount);
    }


    /**
     * Returns the current amount in cents.
     * @return the currents amount of cents as an integer.
     */
    public int amount(){
        return this.amount;
    }

    /**
     * Returns the full name (first name, last name) of the account user.
     * @return The user's full name as a String object.
     */
    public String fullname(){
        return this.owner;
    }

    private static void checkPositiveOrThrow(int i){
        if(i < 0){
            throw new RuntimeException("Amount to transfer should be positive");
        }
    }

    private static void checkOverflowOrThrow(int amount, int amountToAdd){
        if(amount + amountToAdd > Integer.MAX_VALUE){
            throw new RuntimeException("An amount should not overflow");
        }
    }

    private void checkAccountLimit(int amountToRemove){
        if (this.amount - amountToRemove <= this.AMOUNT_LIMIT){
            throw new RuntimeException("Unable to process transaction, amount should never be under " + this.AMOUNT_LIMIT);
        }
    }

    public void deposit(int amountToDeposit){
        Account.checkPositiveOrThrow(amountToDeposit);
        Account.checkOverflowOrThrow(this.amount, amountToDeposit);
        this.amount += amountToDeposit;
    }

    public void withdraw(int amountToWithdraw){
        Account.checkPositiveOrThrow(amountToWithdraw);
        checkAccountLimit(amountToWithdraw);
        this.amount -= amountToWithdraw;
    }

    public void transferTo(Account destination, int amountToTransfer){
        this.withdraw(amountToTransfer);
        destination.deposit(amountToTransfer);
    }

    public String toString(){
        int coins = this.amount / 100;
        int cents = this.amount % 100;
        return this.owner + " : " +  "CHF" + coins + "," + cents;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || obj.getClass() != this.getClass()) {
            return false;
        }

        Account other = (Account) obj;
        return (this.amount == other.amount) && (this.owner.equals(other.owner));
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.owner, this.amount);
    }

}